from roentgen.absorption import MassAttenuationCoefficient
import numpy as np
import astropy.units as u
import matplotlib.pyplot as plt

cdte_atten = MassAttenuationCoefficient('cdte')

energy = u.Quantity(np.arange(1, 1000), 'keV')
atten = cdte_atten.func(energy)

plt.plot(energy, atten)
plt.plot(cdte_atten.energy, cdte_atten.data, 'o')
plt.yscale('log')
plt.xscale('log')
plt.xlabel('Energy [' + str(energy.unit) + ']')
plt.ylabel('Mass attenuation Coefficient [' + str(atten.unit) + ']')
plt.title(cdte_atten.name)
plt.show()



# Licensed under a 3-clause BSD style license - see LICENSE.rst

"""
A Python package for the quantitative analysis of the interaction of energetic photons with matter (x-rays and gamma-rays).
"""
import os
from astropy.io import ascii
from astropy.table import QTable
import astropy.units as u
# Affiliated packages may add whatever they like to this file, but
# should keep this content at the top.
# ----------------------------------------------------------------------------
from ._astropy_init import *
# ----------------------------------------------------------------------------

# For egg_info test builds to pass, put package imports here.
if not _ASTROPY_SETUP_:
    _package_directory = os.path.dirname(os.path.abspath(__file__))
    _data_directory = os.path.abspath(os.path.join(_package_directory, 'data'))

    elements = ascii.read(os.path.join(_data_directory, 'elements.csv'), format='csv', fast_reader=False)

    elemental_densities = QTable(ascii.read(os.path.join(_data_directory, 'elements_densities.csv')))
    elemental_densities['density'] = elemental_densities['density'] * u.g / (u.cm ** 3)
    #elemental_densities.add_index('symbol')

    compounds = QTable(ascii.read(os.path.join(_data_directory, 'compounds_mixtures.csv'), format='csv', fast_reader=False))
    compounds['density'] = compounds['density'] * u.g / (u.cm ** 3)
    #compounds.add_index('symbol')


    emission_energies = QTable(ascii.read(os.path.join(_data_directory, 'emission_energies.csv')))
    for colname in emission_energies.colnames[2:]:
        emission_energies[colname] = emission_energies[colname] * u.eV




"""
"""

from __future__ import absolute_import

import numpy as np
import os
import astropy.units as u
from scipy import interpolate
import roentgen

__all__ = ['Material', 'MassAttenuationCoefficient', 'Compound', 'is_an_element', 'get_atomic_number']

_package_directory = roentgen._package_directory
_data_directory = roentgen._data_directory


class Material(object):
    """An object which provides the properties of a material in x-rays

    Parameters
    ----------
    material_str : str or int
        A representation of the material which includes  (e.g. cdte, be, mylar, si)
    thickness : `astropy.units.Quantity`
        The thickness of the material in the optical path.
    density : `astropy.units.Quantity`
        The density of the material. If None use default values.

    Examples
    --------
    >>> from roentgen.absorption.material import Material
    >>> import astropy.units as u
    >>> detector = Material('cdte', 500 * u.um)
    >>> thermal_blankets = Material('mylar', 0.5 * u.mm)
    """

    @u.quantity_input
    def __init__(self, material_str, thickness: u.m, density=None):
        self.name = material_str
        self.thickness = thickness
        self.mass_attenuation_coefficient = MassAttenuationCoefficient(material_str)
        self.name = self.mass_attenuation_coefficient.name
        if density is None:
            if is_an_element(material_str):
                self.density = roentgen.elemental_densities[get_atomic_number(material_str)-1]['density']
            else:
                # not using loc because table indexing is not yet stable
                # self.density = roentgen.compounds.loc[material_str]['density']
                index = list(roentgen.compounds['symbol']).index(material_str)
                self.density = roentgen.compounds[index]['density']
        else:
            self.density = density

    def __repr__(self):
        """Returns a human-readable representation."""
        txt = '<Material ' + str(self.name) + ' (' + str(self.name) + ') '
        txt += str(self.thickness) + ' ' + str(self.density) + '>'
        return txt

    def __add__(self, other):
        if isinstance(other, Material):
            return Compound([self, other])
        elif isinstance(other, Compound):
            return Compound([self] + other.materials)
        else:
            raise ValueError("Can't add <Material> and " + str(other))

    def transmission(self, energy):
        """Provide the transmission fraction (0 to 1).

        Parameters
        ----------
        energy : `astropy.units.Quantity`
            An array of energies in keV
        """
        coefficients = self.mass_attenuation_coefficient.func(energy)
        transmission = np.exp(- coefficients * self.density * self.thickness)
        return transmission

    def absorption(self, energy):
        """Provides the absorption fraction (0 to 1).

        Parameters
        ----------
        energy : `astropy.units.Quantity`
            An array of energies in keV.
        """
        return 1 - self.transmission(energy)


class Compound(object):
    """An object which provides the x-ray properties of a compound (i.e.
     many materials).

    Parameters
    ----------
    materials : list
        A list of Material objects

    Examples
    --------
    >>> from roentgen.absorption.material import Material, Compound
    >>> import astropy.units as u
    >>> detector = Compound([Material('Pt', 5 * u.um), Material('cdte', 500 * u.um)])
    """

    def __init__(self, materials):
        self.materials = materials

    def __add__(self, other):
        if isinstance(other, Material):
            return Compound(self.materials + [other])
        elif isinstance(other, Compound):
            return Compound(self.materials + other.materials)
        else:
            raise ValueError("Can't add <Compound> and " + str(other))

    def __repr__(self):
        """Returns a human-readable representation."""
        txt = '<Compound '
        for material in self.materials:
            txt += str(material)
        return txt + '>'

    def transmission(self, energy):
        """Provide the transmission fraction (0 to 1).

        Parameters
        ----------
        energy : `astropy.units.Quantity`
            An array of energies in keV
        """
        transmission = np.ones(len(energy), dtype=np.float)
        for material in self.materials:
            coefficients = material.mass_attenuation_coefficient.func(energy)
            transmission *= np.exp(- coefficients * material.density * material.thickness)
        return transmission

    def absorption(self, energy):
        """Provides the absorption fraction (0 to 1).

        Parameters
        ----------
        energy : `astropy.units.Quantity`
            An array of energies in keV.
        """
        return 1 - self.transmission(energy)


class MassAttenuationCoefficient(object):
    """
    The mass attenuation coefficient

    Parameters
    ----------
    material : str
        A string representing a material (e.g. cdte, be, mylar, si)
    """
    def __init__(self, material):
        if is_an_element(material):
            atomic_number = get_atomic_number(material)
            datafile_path = os.path.join(_data_directory, 'elements', 'z' + str(atomic_number).zfill(2) + '.csv')
            symbol = roentgen.elements[atomic_number-1]['symbol']
            name = roentgen.elements[atomic_number-1]['name']
        else:
            datafile_path = os.path.join(_data_directory, 'compounds_mixtures', material.lower().replace(' ', '_') + '.csv')
            index = list(roentgen.compounds['symbol']).index(material)
            symbol = roentgen.compounds[index]['symbol']
            name = roentgen.compounds[index]['name']

        data = np.loadtxt(datafile_path, delimiter=',')
        # find the material in our list
        self.symbol = symbol
        self.name = name
        self.energy = u.Quantity(data[:, 0] * 1000, 'keV')
        self.data = u.Quantity(data[:, 1], 'cm^2/g')

        data_energy_kev = np.log10(self.energy.value)
        data_attenuation_coeff = np.log10(self.data.value)
        self._f = interpolate.interp1d(data_energy_kev, data_attenuation_coeff, bounds_error=False, fill_value=0.0)
        self.func = lambda x: u.Quantity(10 ** self._f(np.log10(x.to('keV').value)), 'cm^2/g')

    def get_filename(material_str):
        if len(material_str) <= 2:
            # likely a symbol of an element
            if material_str in list(elements['symbol']):
                return


def is_an_element(element_str):
    """Returns True is the string represents an element"""
    result = False
    if (len(element_str) <= 2) and (element_str in list(roentgen.elements['symbol'])):
            result = True
    else:
        if element_str in list(roentgen.elements['name']):
            result = True
    return result


def get_atomic_number(element_str):
    """Return the atomic number of the element"""
    # check to see if element_str is symbol
    if is_an_element(element_str):
        if len(element_str) <= 2:
            atomic_number = list(roentgen.elements['symbol']).index(element_str) + 1
        else:
            atomic_number = list(roentgen.elements['name']).index(element_str) + 1
    else:
        atomic_number = None
    return atomic_number



#!/usr/bin/env python
# Licensed under a 3-clause BSD style license - see LICENSE.rst

import glob
import os
import sys

import ah_bootstrap
from setuptools import setup

# A dirty hack to get around some early import/configurations ambiguities
if sys.version_info[0] >= 3:
    import builtins
else:
    import __builtin__ as builtins
builtins._ASTROPY_SETUP_ = True

from astropy_helpers.setup_helpers import (register_commands, get_debug_option,
                                           get_package_info)
from astropy_helpers.git_helpers import get_git_devstr
from astropy_helpers.version_helpers import generate_version_py

# Get some values from the setup.cfg
try:
    from ConfigParser import ConfigParser
except ImportError:
    from configparser import ConfigParser

conf = ConfigParser()
conf.read(['setup.cfg'])
metadata = dict(conf.items('metadata'))

PACKAGENAME = metadata.get('package_name', 'packagename')
DESCRIPTION = metadata.get('description', 'Astropy affiliated package')
AUTHOR = metadata.get('author', '')
AUTHOR_EMAIL = metadata.get('author_email', '')
LICENSE = metadata.get('license', 'unknown')
URL = metadata.get('url', 'http://astropy.org')

# order of priority for long_description:
#   (1) set in setup.cfg,
#   (2) load LONG_DESCRIPTION.rst,
#   (3) load README.rst,
#   (4) package docstring
readme_glob = 'README*'
_cfg_long_description = metadata.get('long_description', '')
if _cfg_long_description:
    LONG_DESCRIPTION = _cfg_long_description

elif os.path.exists('LONG_DESCRIPTION.rst'):
    with open('LONG_DESCRIPTION.rst') as f:
        LONG_DESCRIPTION = f.read()

elif len(glob.glob(readme_glob)) > 0:
    with open(glob.glob(readme_glob)[0]) as f:
        LONG_DESCRIPTION = f.read()

else:
    # Get the long description from the package's docstring
    __import__(PACKAGENAME)
    package = sys.modules[PACKAGENAME]
    LONG_DESCRIPTION = package.__doc__

# Store the package name in a built-in variable so it's easy
# to get from other parts of the setup infrastructure
builtins._ASTROPY_PACKAGE_NAME_ = PACKAGENAME

# VERSION should be PEP440 compatible (http://www.python.org/dev/peps/pep-0440)
VERSION = metadata.get('version', '0.1.0')

# Indicates if this version is a release version
RELEASE = 'dev' not in VERSION

if not RELEASE:
    VERSION += get_git_devstr(False)

# Populate the dict of setup command overrides; this should be done before
# invoking any other functionality from distutils since it can potentially
# modify distutils' behavior.
cmdclassd = register_commands(PACKAGENAME, VERSION, RELEASE)

# Freeze build information in version.py
generate_version_py(PACKAGENAME, VERSION, RELEASE,
                    get_debug_option(PACKAGENAME))

# Treat everything in scripts except README* as a script to be installed
scripts = [fname for fname in glob.glob(os.path.join('scripts', '*'))
           if not os.path.basename(fname).startswith('README')]


# Get configuration information from all of the various subpackages.
# See the docstring for setup_helpers.update_package_files for more
# details.
package_info = get_package_info()

# Add the project-global data
package_info['package_data'].setdefault(PACKAGENAME, [])
package_info['package_data'][PACKAGENAME].append('data/*')

# Define entry points for command-line scripts
entry_points = {'console_scripts': []}

entry_point_list = conf.items('entry_points')
for entry_point in entry_point_list:
    entry_points['console_scripts'].append('{0} = {1}'.format(entry_point[0],
                                                              entry_point[1]))

# Include all .c files, recursively, including those generated by
# Cython, since we can not do this in MANIFEST.in with a "dynamic"
# directory name.
c_files = []
for root, dirs, files in os.walk(PACKAGENAME):
    for filename in files:
        if filename.endswith('.c'):
            c_files.append(
                os.path.join(
                    os.path.relpath(root, PACKAGENAME), filename))
package_info['package_data'][PACKAGENAME].extend(c_files)

# Note that requires and provides should not be included in the call to
# ``setup``, since these are now deprecated. See this link for more details:
# https://groups.google.com/forum/#!topic/astropy-dev/urYO8ckB2uM

setup(name=PACKAGENAME,
      version=VERSION,
      description=DESCRIPTION,
      scripts=scripts,
      install_requires=metadata.get('install_requires', 'astropy').strip().split(),
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      license=LICENSE,
      url=URL,
      long_description=LONG_DESCRIPTION,
      cmdclass=cmdclassd,
      zip_safe=False,
      use_2to3=False,
      entry_points=entry_points,
      **package_info
)



import pytest
from roentgen.absorption.material import MassAttenuationCoefficient, Material, Compound, is_an_element
import roentgen.absorption as abs
import roentgen
import numpy as np
import astropy.units as u

all_materials = list(roentgen.elements['symbol']) + list(roentgen.compounds['symbol'])

@pytest.fixture(params=all_materials)
def mass_atten(request):
    return MassAttenuationCoefficient(request.param)


def test_is_an_element_symbol():
    for el in roentgen.elements['symbol']:
        assert(is_an_element(el))


def test_is_an_element_name():
    for el in roentgen.elements['name']:
        assert(is_an_element(el))


def test_mass_atten(mass_atten):
    # check that all materials can load a MassAttenuationCoefficient object
    assert isinstance(mass_atten, MassAttenuationCoefficient)


def test_returns_quantity(mass_atten):
    assert isinstance(mass_atten.func(1 * u.keV), u.Quantity)


def test_number_of_energies(mass_atten):
    energy = u.Quantity(np.arange(1, 1000), 'keV')
    atten = mass_atten.func(energy)
    assert len(energy) == len(atten)


@pytest.fixture(params=all_materials)
def material(request):
    return Material(request.param, 500 * u.micron)


#@pytest.fixture(params=roentgen.material_list.keys())
#def material_dict(request):
#    return roentgen.material_list[request.param]


#def test_has_default_density(material_dict):
    # test if materials have a default density provided
#    assert material_dict.get('density', 'None') != None


def test_material(material):
    assert isinstance(material, Material)


def test_twomaterials_to_compound(material):
    # check that adding two materials returns a compound
    assert isinstance(material + Material('Si', 500 * u.micron), Compound)


def test_threematerials_to_compound(material):
    # check that adding three materials returns a compound
    assert isinstance(Material('Si', 500 * u.micron) +
                      Material('Ge', 500 * u.micron) +
                      Material('cdte', 100 * u.micron), Compound)


@pytest.fixture(params=all_materials)
def thick_material(request):
    return Material(request.param, 500 * u.Mm)


def test_opaque(thick_material):
    # check that extremely large amounts of material mean no transmission
    assert thick_material.transmission(1 * u.keV).value < 1e-6


@pytest.fixture(params=all_materials)
def thin_material(request):
    return Material(request.param, 1e-6 * u.pm)


def test_transparent(thin_material):
    # check that extremely large amounts of material mean no transmission
    assert thin_material.transmission(1 * u.keV) > 0.9



from roentgen.absorption import Material
import numpy as np
import astropy.units as u
import matplotlib.pyplot as plt

thickness = 500 * u.micron

material_list = ['cdte', 'Si', 'Ge']
energy = u.Quantity(np.arange(1, 100, 0.1), 'keV')

for material in material_list:
    mat = Material(material, thickness)
    plt.plot(energy, mat.absorption(energy), label=mat.name)

plt.xlabel('Energy [' + str(energy.unit) + ']')
plt.ylabel('Efficiency')
plt.legend(loc='lower left')
plt.show()
